from __future__ import annotations

import base64
import platform
import sys
from json import dumps, loads
from random import randint
from time import sleep
from typing import TYPE_CHECKING, Any, Mapping
from urllib.parse import urlencode

import requests
from .exceptions import Auth0Error, RateLimitError
from .types import RequestData, TimeoutType

if TYPE_CHECKING:
    from .rest_async import RequestsResponse

UNKNOWN_ERROR = "a0.sdk.internal.unknown"


class RestClientOptions:
    """Configuration object for RestClient. Used for configuring
            additional RestClient options, such as rate-limit
            retries.

    Args:
        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)
        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
        retries (integer): In the event an API request returns a
            429 response header (indicating rate-limit has been
            hit), the RestClient will retry the request this many
            times using an exponential backoff strategy, before
            raising a RateLimitError exception. 10 retries max.
            (defaults to 3)
        client_info (dict, optional): Custom telemetry data to send
            in the Auth0-Client header instead of the default SDK
            info. Useful for wrapper SDKs that need to identify
            themselves. When provided, this dict is JSON-encoded
            and base64-encoded as the header value. Ignored when
            telemetry is False.
            (defaults to None)
    """

    def __init__(
        self,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        retries: int = 3,
        client_info: dict[str, Any] | None = None,
    ) -> None:
        self.telemetry = telemetry
        self.timeout = timeout
        self.retries = retries
        self.client_info = client_info


class RestClient:
    """Provides simple methods for handling all RESTful api endpoints.

    Args:
        jwt (str, optional): The JWT to be used with the RestClient.
        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)
        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
        options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries. Overrides matching
            options passed to the constructor.
            (defaults to 3)
    """

    def __init__(
        self,
        jwt: str | None,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        options: RestClientOptions | None = None,
    ) -> None:
        if options is None:
            options = RestClientOptions(telemetry=telemetry, timeout=timeout)

        self.options = options
        self.jwt = jwt

        self._metrics = {"retries": 0, "backoff": []}
        self._skip_sleep = False

        self.base_headers = {
            "Content-Type": "application/json",
        }

        if jwt is not None:
            self.base_headers["Authorization"] = f"Bearer {self.jwt}"

        if options.telemetry:
            py_version = platform.python_version()

            if options.client_info is not None:
                auth0_client_dict = options.client_info
            else:
                version = sys.modules["auth0"].__version__
                auth0_client_dict = {
                    "name": "auth0-python",
                    "version": version,
                    "env": {
                        "python": py_version,
                    },
                }

            auth0_client = dumps(auth0_client_dict).encode("utf-8")

            self.base_headers.update(
                {
                    "User-Agent": f"Python/{py_version}",
                    "Auth0-Client": base64.b64encode(auth0_client).decode(),
                }
            )

        # Cap the maximum number of retries to 10 or fewer. Floor the retries at 0.
        self._retries = min(self.MAX_REQUEST_RETRIES(), max(0, options.retries))

        # For backwards compatibility reasons only
        # TODO: Deprecate in the next major so we can prune these arguments. Guidance should be to use RestClient.options.*
        self.telemetry = options.telemetry
        self.timeout = options.timeout

    # Returns a hard cap for the maximum number of retries allowed (10)
    def MAX_REQUEST_RETRIES(self) -> int:
        return 10

    # Returns the maximum amount of jitter to introduce in milliseconds (100ms)
    def MAX_REQUEST_RETRY_JITTER(self) -> int:
        return 100

    # Returns the maximum delay window allowed (1000ms)
    def MAX_REQUEST_RETRY_DELAY(self) -> int:
        return 1000

    # Returns the minimum delay window allowed (100ms)
    def MIN_REQUEST_RETRY_DELAY(self) -> int:
        return 100

    def _request(
        self,
        method: str,
        url: str,
        params: dict[str, Any] | None = None,
        data: RequestData | None = None,
        json: RequestData | None = None,
        headers: dict[str, str] | None = None,
        files: dict[str, Any] | None = None,
    ) -> Any:
        # Track the API request attempt number
        attempt = 0

        # Reset the metrics tracker
        self._metrics = {"retries": 0, "backoff": []}

        if data is None and json is not None and headers:
            content_type = headers.get("Content-Type", "").lower()  # Get Content-Type 
            if "application/x-www-form-urlencoded" in content_type:
                data = urlencode(json)  # Copy JSON data into data
                json = None  # Prevent JSON from being sent

        kwargs = {
            k: v
            for k, v in {
                "params": params,
                "json": json,
                "data": data,
                "headers": headers,
                "files": files,
                "timeout": self.options.timeout,
            }.items()
            if v is not None
        }

        while True:
            # Increment attempt number
            attempt += 1

            # Issue the request
            response = requests.request(method, url, **kwargs)

            # If the response did not have a 429 header, or the attempt number is greater than the configured retries, break
            if response.status_code != 429 or attempt > self._retries:
                break

            wait = self._calculate_wait(attempt)

            # Skip calling sleep() when running unit tests
            if self._skip_sleep is False:
                # sleep() functions in seconds, so convert the milliseconds formula above accordingly
                sleep(wait / 1000)

        # Return the final Response
        return self._process_response(response)

    def get(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        headers: dict[str, str] | None = None,
    ) -> Any:
        request_headers = self.base_headers.copy()
        request_headers.update(headers or {})
        return self._request("GET", url, params=params, headers=request_headers)

    def post(
        self,
        url: str,
        data: RequestData | None = None,
        headers: dict[str, str] | None = None,
    ) -> Any:
        request_headers = self.base_headers.copy()
        request_headers.update(headers or {})
        return self._request("POST", url, json=data, headers=request_headers)

    def file_post(
        self,
        url: str,
        data: RequestData | None = None,
        files: dict[str, Any] | None = None,
    ) -> Any:
        headers = self.base_headers.copy()
        headers.pop("Content-Type", None)
        return self._request("POST", url, data=data, files=files, headers=headers)

    def patch(self, url: str, data: RequestData | None = None) -> Any:
        headers = self.base_headers.copy()
        return self._request("PATCH", url, json=data, headers=headers)

    def put(self, url: str, data: RequestData | None = None) -> Any:
        headers = self.base_headers.copy()
        return self._request("PUT", url, json=data, headers=headers)

    def delete(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: RequestData | None = None,
    ) -> Any:
        headers = self.base_headers.copy()
        return self._request("DELETE", url, params=params, json=data, headers=headers)

    def _calculate_wait(self, attempt: int) -> int:
        # Retry the request. Apply a exponential backoff for subsequent attempts, using this formula:
        # max(MIN_REQUEST_RETRY_DELAY, min(MAX_REQUEST_RETRY_DELAY, (100ms * (2 ** attempt - 1)) + random_between(1, MAX_REQUEST_RETRY_JITTER)))

        # Increases base delay by (100ms * (2 ** attempt - 1))
        wait = 100 * 2 ** (attempt - 1)

        # Introduces jitter to the base delay; increases delay between 1ms to MAX_REQUEST_RETRY_JITTER (100ms)
        wait += randint(1, self.MAX_REQUEST_RETRY_JITTER())

        # Is never more than MAX_REQUEST_RETRY_DELAY (1s)
        wait = min(self.MAX_REQUEST_RETRY_DELAY(), wait)

        # Is never less than MIN_REQUEST_RETRY_DELAY (100ms)
        wait = max(self.MIN_REQUEST_RETRY_DELAY(), wait)

        self._metrics["retries"] = attempt
        self._metrics["backoff"].append(wait)  # type: ignore[attr-defined]

        return wait

    def _process_response(self, response: requests.Response) -> Any:
        return self._parse(response).content()

    def _parse(self, response: requests.Response) -> Response:
        if not response.text:
            return EmptyResponse(response.status_code)
        try:
            return JsonResponse(response)
        except ValueError:
            return PlainResponse(response)


class Response:
    def __init__(
        self, status_code: int, content: Any, headers: Mapping[str, str]
    ) -> None:
        self._status_code = status_code
        self._content = content
        self._headers = headers

    def content(self) -> Any:
        if self._is_error():
            if self._status_code == 429:
                reset_at = int(self._headers.get("x-ratelimit-reset", "-1"))
                raise RateLimitError(
                    error_code=self._error_code(),
                    message=self._error_message(),
                    reset_at=reset_at,
                    headers=self._headers,
                )
            if self._error_code() == "mfa_required":
                raise Auth0Error(
                    status_code=self._status_code,
                    error_code=self._error_code(),
                    message=self._error_message(),
                    content=self._content,
                    headers=self._headers
                )

            raise Auth0Error(
                status_code=self._status_code,
                error_code=self._error_code(),
                message=self._error_message(),
                headers=self._headers
            )
        else:
            return self._content

    def _is_error(self) -> bool:
        return self._status_code is None or self._status_code >= 400

    # Adding these methods to force implementation in subclasses because they are references in this parent class
    def _error_code(self):
        raise NotImplementedError

    def _error_message(self):
        raise NotImplementedError


class JsonResponse(Response):
    def __init__(self, response: requests.Response | RequestsResponse) -> None:
        content = loads(response.text)
        super().__init__(response.status_code, content, response.headers)

    def _error_code(self) -> str:
        if "errorCode" in self._content:
            return self._content.get("errorCode")
        elif "error" in self._content:
            return self._content.get("error")
        elif "code" in self._content:
            return self._content.get("code")
        else:
            return UNKNOWN_ERROR

    def _error_message(self) -> str:
        if "error_description" in self._content:
            return self._content.get("error_description")
        message = self._content.get("message", "")
        if message is not None and message != "":
            return message
        return self._content.get("error", "")


class PlainResponse(Response):
    def __init__(self, response: requests.Response | RequestsResponse) -> None:
        super().__init__(response.status_code, response.text, response.headers)

    def _error_code(self) -> str:
        return UNKNOWN_ERROR

    def _error_message(self) -> str:
        return self._content


class EmptyResponse(Response):
    def __init__(self, status_code: int) -> None:
        super().__init__(status_code, "", {})

    def _error_code(self) -> str:
        return UNKNOWN_ERROR

    def _error_message(self) -> str:
        return ""



# This file was auto-generated by Fern from our API Definition.

AculHeadTagContent = str
"""
Text or markup between the element’s opening and closing tags. 
You can use <a href="https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens">context variables</a> to display dynamic values.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .password_character_type_enum import PasswordCharacterTypeEnum
from .password_character_type_rule_policy_enum import PasswordCharacterTypeRulePolicyEnum
from .password_identical_characters_policy_enum import PasswordIdenticalCharactersPolicyEnum
from .password_max_length_exceeded_policy_enum import PasswordMaxLengthExceededPolicyEnum
from .password_sequential_characters_policy_enum import PasswordSequentialCharactersPolicyEnum


class ConnectionPasswordOptionsComplexity(UniversalBaseModel):
    """
    Password complexity requirements configuration
    """

    min_length: typing.Optional[int] = pydantic.Field(default=None)
    """
    Minimum password length required (1-72 characters)
    """

    character_types: typing.Optional[typing.List[PasswordCharacterTypeEnum]] = pydantic.Field(default=None)
    """
    Required character types that must be present in passwords. Valid options: uppercase, lowercase, number, special
    """

    character_type_rule: typing.Optional[PasswordCharacterTypeRulePolicyEnum] = None
    identical_characters: typing.Optional[PasswordIdenticalCharactersPolicyEnum] = None
    sequential_characters: typing.Optional[PasswordSequentialCharactersPolicyEnum] = None
    max_length_exceeded: typing.Optional[PasswordMaxLengthExceededPolicyEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionRequireRequestUriRegistration = bool
"""
Boolean value specifying whether the OP requires use of the request_uri parameter. If omitted, the default value is false.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.create_encryption_key_public_wrapping_response_content import (
    CreateEncryptionKeyPublicWrappingResponseContent,
)
from ...types.create_encryption_key_response_content import CreateEncryptionKeyResponseContent
from ...types.create_encryption_key_type import CreateEncryptionKeyType
from ...types.encryption_key import EncryptionKey
from ...types.get_encryption_key_response_content import GetEncryptionKeyResponseContent
from ...types.import_encryption_key_response_content import ImportEncryptionKeyResponseContent
from ...types.list_encryption_key_offset_paginated_response_content import (
    ListEncryptionKeyOffsetPaginatedResponseContent,
)
from .raw_client import AsyncRawEncryptionClient, RawEncryptionClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EncryptionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEncryptionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEncryptionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEncryptionClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[EncryptionKey, ListEncryptionKeyOffsetPaginatedResponseContent]:
        """
        Retrieve details of all the encryption keys associated with your tenant.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Default value is 50, maximum value is 100.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[EncryptionKey, ListEncryptionKeyOffsetPaginatedResponseContent]
            The keys were successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.keys.encryption.list(
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def create(
        self, *, type: CreateEncryptionKeyType, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateEncryptionKeyResponseContent:
        """
        Create the new, pre-activated encryption key, without the key material.

        Parameters
        ----------
        type : CreateEncryptionKeyType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEncryptionKeyResponseContent
            The key was successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.encryption.create(
            type="customer-provided-root-key",
        )
        """
        _response = self._raw_client.create(type=type, request_options=request_options)
        return _response.data

    def rekey(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Perform rekeying operation on the key hierarchy.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.encryption.rekey()
        """
        _response = self._raw_client.rekey(request_options=request_options)
        return _response.data

    def get(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetEncryptionKeyResponseContent:
        """
        Retrieve details of the encryption key with the given ID.

        Parameters
        ----------
        kid : str
            Encryption key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEncryptionKeyResponseContent
            The key was successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.encryption.get(
            kid="kid",
        )
        """
        _response = self._raw_client.get(kid, request_options=request_options)
        return _response.data

    def import_(
        self, kid: str, *, wrapped_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ImportEncryptionKeyResponseContent:
        """
        Import wrapped key material and activate encryption key.

        Parameters
        ----------
        kid : str
            Encryption key ID

        wrapped_key : str
            Base64 encoded ciphertext of key material wrapped by public wrapping key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportEncryptionKeyResponseContent
            The key was successfully imported.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.encryption.import_(
            kid="kid",
            wrapped_key="wrapped_key",
        )
        """
        _response = self._raw_client.import_(kid, wrapped_key=wrapped_key, request_options=request_options)
        return _response.data

    def delete(self, kid: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the custom provided encryption key with the given ID and move back to using native encryption key.

        Parameters
        ----------
        kid : str
            Encryption key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.encryption.delete(
            kid="kid",
        )
        """
        _response = self._raw_client.delete(kid, request_options=request_options)
        return _response.data

    def create_public_wrapping_key(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateEncryptionKeyPublicWrappingResponseContent:
        """
        Create the public wrapping key to wrap your own encryption key material.

        Parameters
        ----------
        kid : str
            Encryption key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEncryptionKeyPublicWrappingResponseContent
            The public wrapping key was successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.encryption.create_public_wrapping_key(
            kid="kid",
        )
        """
        _response = self._raw_client.create_public_wrapping_key(kid, request_options=request_options)
        return _response.data


class AsyncEncryptionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEncryptionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEncryptionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEncryptionClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[EncryptionKey, ListEncryptionKeyOffsetPaginatedResponseContent]:
        """
        Retrieve details of all the encryption keys associated with your tenant.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Default value is 50, maximum value is 100.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[EncryptionKey, ListEncryptionKeyOffsetPaginatedResponseContent]
            The keys were successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.keys.encryption.list(
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def create(
        self, *, type: CreateEncryptionKeyType, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateEncryptionKeyResponseContent:
        """
        Create the new, pre-activated encryption key, without the key material.

        Parameters
        ----------
        type : CreateEncryptionKeyType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEncryptionKeyResponseContent
            The key was successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.encryption.create(
                type="customer-provided-root-key",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(type=type, request_options=request_options)
        return _response.data

    async def rekey(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Perform rekeying operation on the key hierarchy.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.encryption.rekey()


        asyncio.run(main())
        """
        _response = await self._raw_client.rekey(request_options=request_options)
        return _response.data

    async def get(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetEncryptionKeyResponseContent:
        """
        Retrieve details of the encryption key with the given ID.

        Parameters
        ----------
        kid : str
            Encryption key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEncryptionKeyResponseContent
            The key was successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.encryption.get(
                kid="kid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(kid, request_options=request_options)
        return _response.data

    async def import_(
        self, kid: str, *, wrapped_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ImportEncryptionKeyResponseContent:
        """
        Import wrapped key material and activate encryption key.

        Parameters
        ----------
        kid : str
            Encryption key ID

        wrapped_key : str
            Base64 encoded ciphertext of key material wrapped by public wrapping key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportEncryptionKeyResponseContent
            The key was successfully imported.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.encryption.import_(
                kid="kid",
                wrapped_key="wrapped_key",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.import_(kid, wrapped_key=wrapped_key, request_options=request_options)
        return _response.data

    async def delete(self, kid: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the custom provided encryption key with the given ID and move back to using native encryption key.

        Parameters
        ----------
        kid : str
            Encryption key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.encryption.delete(
                kid="kid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(kid, request_options=request_options)
        return _response.data

    async def create_public_wrapping_key(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateEncryptionKeyPublicWrappingResponseContent:
        """
        Create the public wrapping key to wrap your own encryption key material.

        Parameters
        ----------
        kid : str
            Encryption key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEncryptionKeyPublicWrappingResponseContent
            The public wrapping key was successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.encryption.create_public_wrapping_key(
                kid="kid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_public_wrapping_key(kid, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.pagination import AsyncPager, SyncPager
from ....core.request_options import RequestOptions
from ....types.list_organization_member_roles_offset_paginated_response_content import (
    ListOrganizationMemberRolesOffsetPaginatedResponseContent,
)
from ....types.role import Role
from .raw_client import AsyncRawRolesClient, RawRolesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RolesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRolesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRolesClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        user_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]:
        """
        Retrieve detailed list of roles assigned to a given user within the context of a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action only returns the roles associated with the specified Organization; any roles assigned to the user within other Organizations are not included.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]
            Roles successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.members.roles.list(
            id="id",
            user_id="user_id",
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id, user_id, page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def assign(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Assign one or more [roles](https://auth0.com/docs/manage-users/access-control/rbac) to a user to determine their access for a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action assigns roles to a user only for the specified Organization. Roles cannot be assigned to a user across multiple Organizations in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        roles : typing.Sequence[str]
            List of roles IDs to associated with the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.members.roles.assign(
            id="id",
            user_id="user_id",
            roles=["roles"],
        )
        """
        _response = self._raw_client.assign(id, user_id, roles=roles, request_options=request_options)
        return _response.data

    def delete(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Remove one or more Organization-specific [roles](https://auth0.com/docs/manage-users/access-control/rbac) from a given user.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action removes roles from a user in relation to the specified Organization. Roles assigned to the user within a different Organization cannot be managed in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            User ID of the organization member to remove roles from.

        roles : typing.Sequence[str]
            List of roles IDs associated with the organization member to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.members.roles.delete(
            id="id",
            user_id="user_id",
            roles=["roles"],
        )
        """
        _response = self._raw_client.delete(id, user_id, roles=roles, request_options=request_options)
        return _response.data


class AsyncRolesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRolesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRolesClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        user_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]:
        """
        Retrieve detailed list of roles assigned to a given user within the context of a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action only returns the roles associated with the specified Organization; any roles assigned to the user within other Organizations are not included.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Role, ListOrganizationMemberRolesOffsetPaginatedResponseContent]
            Roles successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.organizations.members.roles.list(
                id="id",
                user_id="user_id",
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id, user_id, page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def assign(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Assign one or more [roles](https://auth0.com/docs/manage-users/access-control/rbac) to a user to determine their access for a specific Organization.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action assigns roles to a user only for the specified Organization. Roles cannot be assigned to a user across multiple Organizations in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to associate roles with.

        roles : typing.Sequence[str]
            List of roles IDs to associated with the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.members.roles.assign(
                id="id",
                user_id="user_id",
                roles=["roles"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.assign(id, user_id, roles=roles, request_options=request_options)
        return _response.data

    async def delete(
        self,
        id: str,
        user_id: str,
        *,
        roles: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Remove one or more Organization-specific [roles](https://auth0.com/docs/manage-users/access-control/rbac) from a given user.

        Users can be members of multiple Organizations with unique roles assigned for each membership. This action removes roles from a user in relation to the specified Organization. Roles assigned to the user within a different Organization cannot be managed in the same call.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            User ID of the organization member to remove roles from.

        roles : typing.Sequence[str]
            List of roles IDs associated with the organization member to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.members.roles.delete(
                id="id",
                user_id="user_id",
                roles=["roles"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, user_id, roles=roles, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.daily_stats import DailyStats
from ..types.get_active_users_count_stats_response_content import GetActiveUsersCountStatsResponseContent
from pydantic import ValidationError


class RawStatsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_active_users_count(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetActiveUsersCountStatsResponseContent]:
        """
        Retrieve the number of active users that logged in during the last 30 days.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetActiveUsersCountStatsResponseContent]
            Number of active users successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "stats/active-users",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActiveUsersCountStatsResponseContent,
                    parse_obj_as(
                        type_=GetActiveUsersCountStatsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_daily(
        self,
        *,
        from_: typing.Optional[str] = None,
        to: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[DailyStats]]:
        """
        Retrieve the number of logins, signups and breached-password detections (subscription required) that occurred each day within a specified date range.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional first day of the date range (inclusive) in YYYYMMDD format.

        to : typing.Optional[str]
            Optional last day of the date range (inclusive) in YYYYMMDD format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[DailyStats]]
            Daily stats successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "stats/daily",
            method="GET",
            params={
                "from": from_,
                "to": to,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[DailyStats],
                    parse_obj_as(
                        type_=typing.List[DailyStats],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawStatsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_active_users_count(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetActiveUsersCountStatsResponseContent]:
        """
        Retrieve the number of active users that logged in during the last 30 days.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetActiveUsersCountStatsResponseContent]
            Number of active users successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "stats/active-users",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActiveUsersCountStatsResponseContent,
                    parse_obj_as(
                        type_=GetActiveUsersCountStatsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_daily(
        self,
        *,
        from_: typing.Optional[str] = None,
        to: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[DailyStats]]:
        """
        Retrieve the number of logins, signups and breached-password detections (subscription required) that occurred each day within a specified date range.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional first day of the date range (inclusive) in YYYYMMDD format.

        to : typing.Optional[str]
            Optional last day of the date range (inclusive) in YYYYMMDD format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[DailyStats]]
            Daily stats successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "stats/daily",
            method="GET",
            params={
                "from": from_,
                "to": to,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[DailyStats],
                    parse_obj_as(
                        type_=typing.List[DailyStats],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionEnabledClients = typing.List[str]
"""
DEPRECATED property. Use the PATCH /v2/connections/{id}/clients endpoint to enable the connection for a set of clients.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ClientAppTypeEnum = typing.Union[
    typing.Literal[
        "native",
        "spa",
        "regular_web",
        "non_interactive",
        "resource_server",
        "express_configuration",
        "b2b_integration",
        "rms",
        "box",
        "cloudbees",
        "concur",
        "dropbox",
        "mscrm",
        "echosign",
        "egnyte",
        "newrelic",
        "office365",
        "salesforce",
        "sentry",
        "sharepoint",
        "slack",
        "springcm",
        "zendesk",
        "zoom",
        "sso_integration",
        "oag",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

AculHeadTagAttributes = typing.Dict[str, typing.Any]
"""
Attributes of the HTML tag. See <a href="https://developer.mozilla.org/en-US/docs/Web/HTML/Reference/Attributes"> MDN documentation</a> for valid attributes.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TokenQuotaClientCredentials(UniversalBaseModel):
    """
    The token quota configuration
    """

    enforce: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If enabled, the quota will be enforced and requests in excess of the quota will fail. If disabled, the quota will not be enforced, but notifications for requests exceeding the quota will be available in logs.
    """

    per_day: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum number of issued tokens per day
    """

    per_hour: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum number of issued tokens per hour
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import encode_path_param
from ....core.pagination import AsyncPager, SyncPager
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....errors.bad_request_error import BadRequestError
from ....errors.conflict_error import ConflictError
from ....errors.forbidden_error import ForbiddenError
from ....errors.not_found_error import NotFoundError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.create_verifiable_credential_template_response_content import (
    CreateVerifiableCredentialTemplateResponseContent,
)
from ....types.get_verifiable_credential_template_response_content import GetVerifiableCredentialTemplateResponseContent
from ....types.list_verifiable_credential_templates_paginated_response_content import (
    ListVerifiableCredentialTemplatesPaginatedResponseContent,
)
from ....types.mdl_presentation_request import MdlPresentationRequest
from ....types.update_verifiable_credential_template_response_content import (
    UpdateVerifiableCredentialTemplateResponseContent,
)
from ....types.verifiable_credential_template_response import VerifiableCredentialTemplateResponse
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]:
        """
        List verifiable credential templates.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]
            Templates successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "verifiable-credentials/verification/templates",
            method="GET",
            params={
                "from": from_,
                "take": take,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListVerifiableCredentialTemplatesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListVerifiableCredentialTemplatesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.templates
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    from_=_parsed_next,
                    take=take,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        type: str,
        dialect: str,
        presentation: MdlPresentationRequest,
        well_known_trusted_issuers: str,
        custom_certificate_authority: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateVerifiableCredentialTemplateResponseContent]:
        """
        Create a verifiable credential template.

        Parameters
        ----------
        name : str

        type : str

        dialect : str

        presentation : MdlPresentationRequest

        well_known_trusted_issuers : str

        custom_certificate_authority : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateVerifiableCredentialTemplateResponseContent]
            Template successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "verifiable-credentials/verification/templates",
            method="POST",
            json={
                "name": name,
                "type": type,
                "dialect": dialect,
                "presentation": convert_and_respect_annotation_metadata(
                    object_=presentation, annotation=MdlPresentationRequest, direction="write"
                ),
                "custom_certificate_authority": custom_certificate_authority,
                "well_known_trusted_issuers": well_known_trusted_issuers,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateVerifiableCredentialTemplateResponseContent,
                    parse_obj_as(
                        type_=CreateVerifiableCredentialTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetVerifiableCredentialTemplateResponseContent]:
        """
        Get a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetVerifiableCredentialTemplateResponseContent]
            Template successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"verifiable-credentials/verification/templates/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetVerifiableCredentialTemplateResponseContent,
                    parse_obj_as(
                        type_=GetVerifiableCredentialTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"verifiable-credentials/verification/templates/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        type: typing.Optional[str] = OMIT,
        dialect: typing.Optional[str] = OMIT,
        presentation: typing.Optional[MdlPresentationRequest] = OMIT,
        well_known_trusted_issuers: typing.Optional[str] = OMIT,
        version: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateVerifiableCredentialTemplateResponseContent]:
        """
        Update a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        name : typing.Optional[str]

        type : typing.Optional[str]

        dialect : typing.Optional[str]

        presentation : typing.Optional[MdlPresentationRequest]

        well_known_trusted_issuers : typing.Optional[str]

        version : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateVerifiableCredentialTemplateResponseContent]
            Template successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"verifiable-credentials/verification/templates/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "type": type,
                "dialect": dialect,
                "presentation": convert_and_respect_annotation_metadata(
                    object_=presentation, annotation=MdlPresentationRequest, direction="write"
                ),
                "well_known_trusted_issuers": well_known_trusted_issuers,
                "version": version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateVerifiableCredentialTemplateResponseContent,
                    parse_obj_as(
                        type_=UpdateVerifiableCredentialTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]:
        """
        List verifiable credential templates.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]
            Templates successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "verifiable-credentials/verification/templates",
            method="GET",
            params={
                "from": from_,
                "take": take,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListVerifiableCredentialTemplatesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListVerifiableCredentialTemplatesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.templates
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        from_=_parsed_next,
                        take=take,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        type: str,
        dialect: str,
        presentation: MdlPresentationRequest,
        well_known_trusted_issuers: str,
        custom_certificate_authority: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateVerifiableCredentialTemplateResponseContent]:
        """
        Create a verifiable credential template.

        Parameters
        ----------
        name : str

        type : str

        dialect : str

        presentation : MdlPresentationRequest

        well_known_trusted_issuers : str

        custom_certificate_authority : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateVerifiableCredentialTemplateResponseContent]
            Template successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "verifiable-credentials/verification/templates",
            method="POST",
            json={
                "name": name,
                "type": type,
                "dialect": dialect,
                "presentation": convert_and_respect_annotation_metadata(
                    object_=presentation, annotation=MdlPresentationRequest, direction="write"
                ),
                "custom_certificate_authority": custom_certificate_authority,
                "well_known_trusted_issuers": well_known_trusted_issuers,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateVerifiableCredentialTemplateResponseContent,
                    parse_obj_as(
                        type_=CreateVerifiableCredentialTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetVerifiableCredentialTemplateResponseContent]:
        """
        Get a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetVerifiableCredentialTemplateResponseContent]
            Template successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"verifiable-credentials/verification/templates/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetVerifiableCredentialTemplateResponseContent,
                    parse_obj_as(
                        type_=GetVerifiableCredentialTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"verifiable-credentials/verification/templates/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        type: typing.Optional[str] = OMIT,
        dialect: typing.Optional[str] = OMIT,
        presentation: typing.Optional[MdlPresentationRequest] = OMIT,
        well_known_trusted_issuers: typing.Optional[str] = OMIT,
        version: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateVerifiableCredentialTemplateResponseContent]:
        """
        Update a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        name : typing.Optional[str]

        type : typing.Optional[str]

        dialect : typing.Optional[str]

        presentation : typing.Optional[MdlPresentationRequest]

        well_known_trusted_issuers : typing.Optional[str]

        version : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateVerifiableCredentialTemplateResponseContent]
            Template successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"verifiable-credentials/verification/templates/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "type": type,
                "dialect": dialect,
                "presentation": convert_and_respect_annotation_metadata(
                    object_=presentation, annotation=MdlPresentationRequest, direction="write"
                ),
                "well_known_trusted_issuers": well_known_trusted_issuers,
                "version": version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateVerifiableCredentialTemplateResponseContent,
                    parse_obj_as(
                        type_=UpdateVerifiableCredentialTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionOptionsFreeformScopesGithub = ConnectionScopeArray
"""
Array of custom OAuth 2.0 scopes to request from GitHub during authentication. Use this to request scopes not covered by the predefined scope options.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ClientMetadata = typing.Dict[str, typing.Any]
"""
Metadata associated with the client, in the form of an object with string values (max 255 chars).  Maximum of 10 metadata properties allowed.  Field names (max 255 chars) are alphanumeric and may only include the following special characters:  :,-+=_*?"/\()<>@    [Tab] [Space]
"""



# This file was auto-generated by Fern from our API Definition.

import typing

FlowActionSalesforceSearchLeadsParamsSearchField = typing.Union[
    typing.Literal["email", "name", "phone", "all"], typing.Any
]



from .conftest import get_client, verify_request_count

from auth0.management import CreateFlowsVaultConnectionActivecampaignApiKey, FlowsVaultConnectioSetupApiKeyWithBaseUrl


def test_flows_vault_connections_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "flows.vault.connections.list_.0"
    client = get_client(test_id)
    client.flows.vault.connections.list(
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id, "GET", "/flows/vault/connections", {"page": "1", "per_page": "1", "include_totals": "true"}, 1
    )


def test_flows_vault_connections_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "flows.vault.connections.create.0"
    client = get_client(test_id)
    client.flows.vault.connections.create(
        request=CreateFlowsVaultConnectionActivecampaignApiKey(
            name="name",
            app_id="ACTIVECAMPAIGN",
            setup=FlowsVaultConnectioSetupApiKeyWithBaseUrl(
                type="API_KEY",
                api_key="api_key",
                base_url="base_url",
            ),
        ),
    )
    verify_request_count(test_id, "POST", "/flows/vault/connections", None, 1)


def test_flows_vault_connections_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "flows.vault.connections.get.0"
    client = get_client(test_id)
    client.flows.vault.connections.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/flows/vault/connections/id", None, 1)


def test_flows_vault_connections_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "flows.vault.connections.delete.0"
    client = get_client(test_id)
    client.flows.vault.connections.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/flows/vault/connections/id", None, 1)


def test_flows_vault_connections_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "flows.vault.connections.update.0"
    client = get_client(test_id)
    client.flows.vault.connections.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/flows/vault/connections/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

GuardianFactorsProviderPushNotificationProviderDataEnum = typing.Union[
    typing.Literal["guardian", "sns", "direct"], typing.Any
]



from .conftest import get_client, verify_request_count


def test_groups_roles_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "groups.roles.list_.0"
    client = get_client(test_id)
    client.groups.roles.list(
        id="id",
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/groups/id/roles", {"from": "from", "take": "1"}, 1)


def test_groups_roles_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "groups.roles.create.0"
    client = get_client(test_id)
    client.groups.roles.create(
        id="id",
        roles=["roles"],
    )
    verify_request_count(test_id, "POST", "/groups/id/roles", None, 1)


def test_groups_roles_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "groups.roles.delete.0"
    client = get_client(test_id)
    client.groups.roles.delete(
        id="id",
        roles=["roles"],
    )
    verify_request_count(test_id, "DELETE", "/groups/id/roles", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....errors.bad_request_error import BadRequestError
from ....errors.forbidden_error import ForbiddenError
from ....errors.not_found_error import NotFoundError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.get_risk_assessments_settings_new_device_response_content import (
    GetRiskAssessmentsSettingsNewDeviceResponseContent,
)
from ....types.update_risk_assessments_settings_new_device_response_content import (
    UpdateRiskAssessmentsSettingsNewDeviceResponseContent,
)
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawNewDeviceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetRiskAssessmentsSettingsNewDeviceResponseContent]:
        """
        Gets the risk assessment settings for the new device assessor

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetRiskAssessmentsSettingsNewDeviceResponseContent]
            Returning risk assessment settings for new devices
        """
        _response = self._client_wrapper.httpx_client.request(
            "risk-assessments/settings/new-device",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetRiskAssessmentsSettingsNewDeviceResponseContent,
                    parse_obj_as(
                        type_=GetRiskAssessmentsSettingsNewDeviceResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, *, remember_for: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateRiskAssessmentsSettingsNewDeviceResponseContent]:
        """
        Updates the risk assessment settings for the new device assessor

        Parameters
        ----------
        remember_for : int
            Length of time to remember devices for, in days.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateRiskAssessmentsSettingsNewDeviceResponseContent]
            Returning risk assessment settings for new devices
        """
        _response = self._client_wrapper.httpx_client.request(
            "risk-assessments/settings/new-device",
            method="PATCH",
            json={
                "remember_for": remember_for,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateRiskAssessmentsSettingsNewDeviceResponseContent,
                    parse_obj_as(
                        type_=UpdateRiskAssessmentsSettingsNewDeviceResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawNewDeviceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetRiskAssessmentsSettingsNewDeviceResponseContent]:
        """
        Gets the risk assessment settings for the new device assessor

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetRiskAssessmentsSettingsNewDeviceResponseContent]
            Returning risk assessment settings for new devices
        """
        _response = await self._client_wrapper.httpx_client.request(
            "risk-assessments/settings/new-device",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetRiskAssessmentsSettingsNewDeviceResponseContent,
                    parse_obj_as(
                        type_=GetRiskAssessmentsSettingsNewDeviceResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, *, remember_for: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateRiskAssessmentsSettingsNewDeviceResponseContent]:
        """
        Updates the risk assessment settings for the new device assessor

        Parameters
        ----------
        remember_for : int
            Length of time to remember devices for, in days.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateRiskAssessmentsSettingsNewDeviceResponseContent]
            Returning risk assessment settings for new devices
        """
        _response = await self._client_wrapper.httpx_client.request(
            "risk-assessments/settings/new-device",
            method="PATCH",
            json={
                "remember_for": remember_for,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateRiskAssessmentsSettingsNewDeviceResponseContent,
                    parse_obj_as(
                        type_=UpdateRiskAssessmentsSettingsNewDeviceResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .linked_client_configuration import LinkedClientConfiguration


class ExpressConfigurationOrNull(UniversalBaseModel):
    """
    Application specific configuration for use with the OIN Express Configuration feature.
    """

    initiate_login_uri_template: str = pydantic.Field()
    """
    The URI users should bookmark to log in to this application. Variable substitution is permitted for the following properties: organization_name, organization_id, and connection_name.
    """

    user_attribute_profile_id: str = pydantic.Field()
    """
    The ID of the user attribute profile to use for this application.
    """

    connection_profile_id: str = pydantic.Field()
    """
    The ID of the connection profile to use for this application.
    """

    enable_client: bool = pydantic.Field()
    """
    When true, all connections made via express configuration will be enabled for this application.
    """

    enable_organization: bool = pydantic.Field()
    """
    When true, all connections made via express configuration will have the associated organization enabled.
    """

    linked_clients: typing.Optional[typing.List[LinkedClientConfiguration]] = pydantic.Field(default=None)
    """
    List of client IDs that are linked to this express configuration (e.g. web or mobile clients).
    """

    okta_oin_client_id: str = pydantic.Field()
    """
    This is the unique identifier for the Okta OIN Express Configuration Client, which Okta will use for this application.
    """

    admin_login_domain: str = pydantic.Field()
    """
    This is the domain that admins are expected to log in via for authenticating for express configuration. It can be either the canonical domain or a registered custom domain.
    """

    oin_submission_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identifier of the published application in the OKTA OIN.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Any

from .base import AuthenticationBase


class RevokeToken(AuthenticationBase):
    """Revoke Refresh Token endpoint

    Args:
        domain (str): Your auth0 domain (e.g: my-domain.us.auth0.com)
    """

    def revoke_refresh_token(self, token: str) -> Any:
        """Revokes a Refresh Token if it has been compromised

        Each revocation request invalidates not only the specific token, but all other tokens
        based on the same authorization grant. This means that all Refresh Tokens that have
        been issued for the same user, application, and audience will be revoked.

        Args:
             token (str): The Refresh Token you want to revoke

         See: https://auth0.com/docs/api/authentication#refresh-token
        """
        body = {
            "client_id": self.client_id,
            "token": token,
        }

        return self.authenticated_post(
            f"{self.protocol}://{self.domain}/oauth/revoke", data=body
        )



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionProfileOrganizationShowAsButtonEnum = typing.Union[typing.Literal["none", "optional", "required"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.logging import LogConfig, Logger
from .environment import Auth0Environment

if typing.TYPE_CHECKING:
    from .actions.client import ActionsClient, AsyncActionsClient
    from .anomaly.client import AnomalyClient, AsyncAnomalyClient
    from .attack_protection.client import AsyncAttackProtectionClient, AttackProtectionClient
    from .branding.client import AsyncBrandingClient, BrandingClient
    from .client_grants.client import AsyncClientGrantsClient, ClientGrantsClient
    from .clients.client import AsyncClientsClient, ClientsClient
    from .connection_profiles.client import AsyncConnectionProfilesClient, ConnectionProfilesClient
    from .connections.client import AsyncConnectionsClient, ConnectionsClient
    from .custom_domains.client import AsyncCustomDomainsClient, CustomDomainsClient
    from .device_credentials.client import AsyncDeviceCredentialsClient, DeviceCredentialsClient
    from .email_templates.client import AsyncEmailTemplatesClient, EmailTemplatesClient
    from .emails.client import AsyncEmailsClient, EmailsClient
    from .event_streams.client import AsyncEventStreamsClient, EventStreamsClient
    from .events.client import AsyncEventsClient, EventsClient
    from .flows.client import AsyncFlowsClient, FlowsClient
    from .forms.client import AsyncFormsClient, FormsClient
    from .groups.client import AsyncGroupsClient, GroupsClient
    from .guardian.client import AsyncGuardianClient, GuardianClient
    from .hooks.client import AsyncHooksClient, HooksClient
    from .jobs.client import AsyncJobsClient, JobsClient
    from .keys.client import AsyncKeysClient, KeysClient
    from .log_streams.client import AsyncLogStreamsClient, LogStreamsClient
    from .logs.client import AsyncLogsClient, LogsClient
    from .network_acls.client import AsyncNetworkAclsClient, NetworkAclsClient
    from .organizations.client import AsyncOrganizationsClient, OrganizationsClient
    from .prompts.client import AsyncPromptsClient, PromptsClient
    from .rate_limit_policies.client import AsyncRateLimitPoliciesClient, RateLimitPoliciesClient
    from .refresh_tokens.client import AsyncRefreshTokensClient, RefreshTokensClient
    from .resource_servers.client import AsyncResourceServersClient, ResourceServersClient
    from .risk_assessments.client import AsyncRiskAssessmentsClient, RiskAssessmentsClient
    from .roles.client import AsyncRolesClient, RolesClient
    from .rules.client import AsyncRulesClient, RulesClient
    from .rules_configs.client import AsyncRulesConfigsClient, RulesConfigsClient
    from .self_service_profiles.client import AsyncSelfServiceProfilesClient, SelfServiceProfilesClient
    from .sessions.client import AsyncSessionsClient, SessionsClient
    from .stats.client import AsyncStatsClient, StatsClient
    from .supplemental_signals.client import AsyncSupplementalSignalsClient, SupplementalSignalsClient
    from .tenants.client import AsyncTenantsClient, TenantsClient
    from .tickets.client import AsyncTicketsClient, TicketsClient
    from .token_exchange_profiles.client import AsyncTokenExchangeProfilesClient, TokenExchangeProfilesClient
    from .user_attribute_profiles.client import AsyncUserAttributeProfilesClient, UserAttributeProfilesClient
    from .user_blocks.client import AsyncUserBlocksClient, UserBlocksClient
    from .user_grants.client import AsyncUserGrantsClient, UserGrantsClient
    from .users.client import AsyncUsersClient, UsersClient
    from .verifiable_credentials.client import AsyncVerifiableCredentialsClient, VerifiableCredentialsClient


class Auth0:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : Auth0Environment
        The environment to use for requests from the client. from .environment import Auth0Environment



        Defaults to Auth0Environment.DEFAULT



    tenant_domain : typing.Optional[str]
        Server URL variable for 'tenantDomain'. Defaults to '{TENANT}.auth0.com'.

    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    max_retries : typing.Optional[int]
        The default maximum number of retries for failed requests. Defaults to 2. Per-request `max_retries` in `request_options` takes precedence over this value.

    stream_reconnection_enabled : typing.Optional[bool]
        Whether to automatically reconnect on stream disconnection for resumable streaming endpoints. Defaults to True. Per-request `stream_reconnection_enabled` in `request_options` takes precedence over this value.

    max_stream_reconnection_attempts : typing.Optional[int]
        The maximum number of reconnection attempts for resumable streaming endpoints. Defaults to no limit. Per-request `max_stream_reconnection_attempts` in `request_options` takes precedence over this value.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    logging : typing.Optional[typing.Union[LogConfig, Logger]]
        Configure logging for the SDK. Accepts a LogConfig dict with 'level' (debug/info/warn/error), 'logger' (custom logger implementation), and 'silent' (boolean, defaults to True) fields. You can also pass a pre-configured Logger instance.

    Examples
    --------
    from auth0 import Auth0

    client = Auth0(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: Auth0Environment = Auth0Environment.DEFAULT,
        tenant_domain: typing.Optional[str] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        max_retries: typing.Optional[int] = None,
        stream_reconnection_enabled: typing.Optional[bool] = None,
        max_stream_reconnection_attempts: typing.Optional[int] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        _defaulted_max_retries = max_retries if max_retries is not None else 2
        if tenant_domain is not None:
            _tenant_domain = tenant_domain if tenant_domain is not None else "{TENANT}.auth0.com"
            base_url = "https://{tenantDomain}/api/v2".format(tenantDomain=_tenant_domain)
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
            max_retries=_defaulted_max_retries,
            stream_reconnection_enabled=stream_reconnection_enabled,
            max_stream_reconnection_attempts=max_stream_reconnection_attempts,
            logging=logging,
        )
        self._actions: typing.Optional[ActionsClient] = None
        self._branding: typing.Optional[BrandingClient] = None
        self._client_grants: typing.Optional[ClientGrantsClient] = None
        self._clients: typing.Optional[ClientsClient] = None
        self._connection_profiles: typing.Optional[ConnectionProfilesClient] = None
        self._connections: typing.Optional[ConnectionsClient] = None
        self._custom_domains: typing.Optional[CustomDomainsClient] = None
        self._device_credentials: typing.Optional[DeviceCredentialsClient] = None
        self._email_templates: typing.Optional[EmailTemplatesClient] = None
        self._event_streams: typing.Optional[EventStreamsClient] = None
        self._events: typing.Optional[EventsClient] = None
        self._flows: typing.Optional[FlowsClient] = None
        self._forms: typing.Optional[FormsClient] = None
        self._user_grants: typing.Optional[UserGrantsClient] = None
        self._groups: typing.Optional[GroupsClient] = None
        self._hooks: typing.Optional[HooksClient] = None
        self._jobs: typing.Optional[JobsClient] = None
        self._log_streams: typing.Optional[LogStreamsClient] = None
        self._logs: typing.Optional[LogsClient] = None
        self._network_acls: typing.Optional[NetworkAclsClient] = None
        self._organizations: typing.Optional[OrganizationsClient] = None
        self._prompts: typing.Optional[PromptsClient] = None
        self._rate_limit_policies: typing.Optional[RateLimitPoliciesClient] = None
        self._refresh_tokens: typing.Optional[RefreshTokensClient] = None
        self._resource_servers: typing.Optional[ResourceServersClient] = None
        self._roles: typing.Optional[RolesClient] = None
        self._rules: typing.Optional[RulesClient] = None
        self._rules_configs: typing.Optional[RulesConfigsClient] = None
        self._self_service_profiles: typing.Optional[SelfServiceProfilesClient] = None
        self._sessions: typing.Optional[SessionsClient] = None
        self._stats: typing.Optional[StatsClient] = None
        self._supplemental_signals: typing.Optional[SupplementalSignalsClient] = None
        self._tickets: typing.Optional[TicketsClient] = None
        self._token_exchange_profiles: typing.Optional[TokenExchangeProfilesClient] = None
        self._user_attribute_profiles: typing.Optional[UserAttributeProfilesClient] = None
        self._user_blocks: typing.Optional[UserBlocksClient] = None
        self._users: typing.Optional[UsersClient] = None
        self._anomaly: typing.Optional[AnomalyClient] = None
        self._attack_protection: typing.Optional[AttackProtectionClient] = None
        self._emails: typing.Optional[EmailsClient] = None
        self._guardian: typing.Optional[GuardianClient] = None
        self._keys: typing.Optional[KeysClient] = None
        self._risk_assessments: typing.Optional[RiskAssessmentsClient] = None
        self._tenants: typing.Optional[TenantsClient] = None
        self._verifiable_credentials: typing.Optional[VerifiableCredentialsClient] = None

    @property
    def actions(self):
        if self._actions is None:
            from .actions.client import ActionsClient  # noqa: E402

            self._actions = ActionsClient(client_wrapper=self._client_wrapper)
        return self._actions

    @property
    def branding(self):
        if self._branding is None:
            from .branding.client import BrandingClient  # noqa: E402

            self._branding = BrandingClient(client_wrapper=self._client_wrapper)
        return self._branding

    @property
    def client_grants(self):
        if self._client_grants is None:
            from .client_grants.client import ClientGrantsClient  # noqa: E402

            self._client_grants = ClientGrantsClient(client_wrapper=self._client_wrapper)
        return self._client_grants

    @property
    def clients(self):
        if self._clients is None:
            from .clients.client import ClientsClient  # noqa: E402

            self._clients = ClientsClient(client_wrapper=self._client_wrapper)
        return self._clients

    @property
    def connection_profiles(self):
        if self._connection_profiles is None:
            from .connection_profiles.client import ConnectionProfilesClient  # noqa: E402

            self._connection_profiles = ConnectionProfilesClient(client_wrapper=self._client_wrapper)
        return self._connection_profiles

    @property
    def connections(self):
        if self._connections is None:
            from .connections.client import ConnectionsClient  # noqa: E402

            self._connections = ConnectionsClient(client_wrapper=self._client_wrapper)
        return self._connections

    @property
    def custom_domains(self):
        if self._custom_domains is None:
            from .custom_domains.client import CustomDomainsClient  # noqa: E402

            self._custom_domains = CustomDomainsClient(client_wrapper=self._client_wrapper)
        return self._custom_domains

    @property
    def device_credentials(self):
        if self._device_credentials is None:
            from .device_credentials.client import DeviceCredentialsClient  # noqa: E402

            self._device_credentials = DeviceCredentialsClient(client_wrapper=self._client_wrapper)
        return self._device_credentials

    @property
    def email_templates(self):
        if self._email_templates is None:
            from .email_templates.client import EmailTemplatesClient  # noqa: E402

            self._email_templates = EmailTemplatesClient(client_wrapper=self._client_wrapper)
        return self._email_templates

    @property
    def event_streams(self):
        if self._event_streams is None:
            from .event_streams.client import EventStreamsClient  # noqa: E402

            self._event_streams = EventStreamsClient(client_wrapper=self._client_wrapper)
        return self._event_streams

    @property
    def events(self):
        if self._events is None:
            from .events.client import EventsClient  # noqa: E402

            self._events = EventsClient(client_wrapper=self._client_wrapper)
        return self._events

    @property
    def flows(self):
        if self._flows is None:
            from .flows.client import FlowsClient  # noqa: E402

            self._flows = FlowsClient(client_wrapper=self._client_wrapper)
        return self._flows

    @property
    def forms(self):
        if self._forms is None:
            from .forms.client import FormsClient  # noqa: E402

            self._forms = FormsClient(client_wrapper=self._client_wrapper)
        return self._forms

    @property
    def user_grants(self):
        if self._user_grants is None:
            from .user_grants.client import UserGrantsClient  # noqa: E402

            self._user_grants = UserGrantsClient(client_wrapper=self._client_wrapper)
        return self._user_grants

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import GroupsClient  # noqa: E402

            self._groups = GroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def hooks(self):
        if self._hooks is None:
            from .hooks.client import HooksClient  # noqa: E402

            self._hooks = HooksClient(client_wrapper=self._client_wrapper)
        return self._hooks

    @property
    def jobs(self):
        if self._jobs is None:
            from .jobs.client import JobsClient  # noqa: E402

            self._jobs = JobsClient(client_wrapper=self._client_wrapper)
        return self._jobs

    @property
    def log_streams(self):
        if self._log_streams is None:
            from .log_streams.client import LogStreamsClient  # noqa: E402

            self._log_streams = LogStreamsClient(client_wrapper=self._client_wrapper)
        return self._log_streams

    @property
    def logs(self):
        if self._logs is None:
            from .logs.client import LogsClient  # noqa: E402

            self._logs = LogsClient(client_wrapper=self._client_wrapper)
        return self._logs

    @property
    def network_acls(self):
        if self._network_acls is None:
            from .network_acls.client import NetworkAclsClient  # noqa: E402

            self._network_acls = NetworkAclsClient(client_wrapper=self._client_wrapper)
        return self._network_acls

    @property
    def organizations(self):
        if self._organizations is None:
            from .organizations.client import OrganizationsClient  # noqa: E402

            self._organizations = OrganizationsClient(client_wrapper=self._client_wrapper)
        return self._organizations

    @property
    def prompts(self):
        if self._prompts is None:
            from .prompts.client import PromptsClient  # noqa: E402

            self._prompts = PromptsClient(client_wrapper=self._client_wrapper)
        return self._prompts

    @property
    def rate_limit_policies(self):
        if self._rate_limit_policies is None:
            from .rate_limit_policies.client import RateLimitPoliciesClient  # noqa: E402

            self._rate_limit_policies = RateLimitPoliciesClient(client_wrapper=self._client_wrapper)
        return self._rate_limit_policies

    @property
    def refresh_tokens(self):
        if self._refresh_tokens is None:
            from .refresh_tokens.client import RefreshTokensClient  # noqa: E402

            self._refresh_tokens = RefreshTokensClient(client_wrapper=self._client_wrapper)
        return self._refresh_tokens

    @property
    def resource_servers(self):
        if self._resource_servers is None:
            from .resource_servers.client import ResourceServersClient  # noqa: E402

            self._resource_servers = ResourceServersClient(client_wrapper=self._client_wrapper)
        return self._resource_servers

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import RolesClient  # noqa: E402

            self._roles = RolesClient(client_wrapper=self._client_wrapper)
        return self._roles

    @property
    def rules(self):
        if self._rules is None:
            from .rules.client import RulesClient  # noqa: E402

            self._rules = RulesClient(client_wrapper=self._client_wrapper)
        return self._rules

    @property
    def rules_configs(self):
        if self._rules_configs is None:
            from .rules_configs.client import RulesConfigsClient  # noqa: E402

            self._rules_configs = RulesConfigsClient(client_wrapper=self._client_wrapper)
        return self._rules_configs

    @property
    def self_service_profiles(self):
        if self._self_service_profiles is None:
            from .self_service_profiles.client import SelfServiceProfilesClient  # noqa: E402

            self._self_service_profiles = SelfServiceProfilesClient(client_wrapper=self._client_wrapper)
        return self._self_service_profiles

    @property
    def sessions(self):
        if self._sessions is None:
            from .sessions.client import SessionsClient  # noqa: E402

            self._sessions = SessionsClient(client_wrapper=self._client_wrapper)
        return self._sessions

    @property
    def stats(self):
        if self._stats is None:
            from .stats.client import StatsClient  # noqa: E402

            self._stats = StatsClient(client_wrapper=self._client_wrapper)
        return self._stats

    @property
    def supplemental_signals(self):
        if self._supplemental_signals is None:
            from .supplemental_signals.client import SupplementalSignalsClient  # noqa: E402

            self._supplemental_signals = SupplementalSignalsClient(client_wrapper=self._client_wrapper)
        return self._supplemental_signals

    @property
    def tickets(self):
        if self._tickets is None:
            from .tickets.client import TicketsClient  # noqa: E402

            self._tickets = TicketsClient(client_wrapper=self._client_wrapper)
        return self._tickets

    @property
    def token_exchange_profiles(self):
        if self._token_exchange_profiles is None:
            from .token_exchange_profiles.client import TokenExchangeProfilesClient  # noqa: E402

            self._token_exchange_profiles = TokenExchangeProfilesClient(client_wrapper=self._client_wrapper)
        return self._token_exchange_profiles

    @property
    def user_attribute_profiles(self):
        if self._user_attribute_profiles is None:
            from .user_attribute_profiles.client import UserAttributeProfilesClient  # noqa: E402

            self._user_attribute_profiles = UserAttributeProfilesClient(client_wrapper=self._client_wrapper)
        return self._user_attribute_profiles

    @property
    def user_blocks(self):
        if self._user_blocks is None:
            from .user_blocks.client import UserBlocksClient  # noqa: E402

            self._user_blocks = UserBlocksClient(client_wrapper=self._client_wrapper)
        return self._user_blocks

    @property
    def users(self):
        if self._users is None:
            from .users.client import UsersClient  # noqa: E402

            self._users = UsersClient(client_wrapper=self._client_wrapper)
        return self._users

    @property
    def anomaly(self):
        if self._anomaly is None:
            from .anomaly.client import AnomalyClient  # noqa: E402

            self._anomaly = AnomalyClient(client_wrapper=self._client_wrapper)
        return self._anomaly

    @property
    def attack_protection(self):
        if self._attack_protection is None:
            from .attack_protection.client import AttackProtectionClient  # noqa: E402

            self._attack_protection = AttackProtectionClient(client_wrapper=self._client_wrapper)
        return self._attack_protection

    @property
    def emails(self):
        if self._emails is None:
            from .emails.client import EmailsClient  # noqa: E402

            self._emails = EmailsClient(client_wrapper=self._client_wrapper)
        return self._emails

    @property
    def guardian(self):
        if self._guardian is None:
            from .guardian.client import GuardianClient  # noqa: E402

            self._guardian = GuardianClient(client_wrapper=self._client_wrapper)
        return self._guardian

    @property
    def keys(self):
        if self._keys is None:
            from .keys.client import KeysClient  # noqa: E402

            self._keys = KeysClient(client_wrapper=self._client_wrapper)
        return self._keys

    @property
    def risk_assessments(self):
        if self._risk_assessments is None:
            from .risk_assessments.client import RiskAssessmentsClient  # noqa: E402

            self._risk_assessments = RiskAssessmentsClient(client_wrapper=self._client_wrapper)
        return self._risk_assessments

    @property
    def tenants(self):
        if self._tenants is None:
            from .tenants.client import TenantsClient  # noqa: E402

            self._tenants = TenantsClient(client_wrapper=self._client_wrapper)
        return self._tenants

    @property
    def verifiable_credentials(self):
        if self._verifiable_credentials is None:
            from .verifiable_credentials.client import VerifiableCredentialsClient  # noqa: E402

            self._verifiable_credentials = VerifiableCredentialsClient(client_wrapper=self._client_wrapper)
        return self._verifiable_credentials


def _make_default_async_client(
    timeout: typing.Optional[float],
    follow_redirects: typing.Optional[bool],
) -> httpx.AsyncClient:
    try:
        import httpx_aiohttp  # type: ignore[import-not-found]
    except ImportError:
        pass
    else:
        if follow_redirects is not None:
            return httpx_aiohttp.HttpxAiohttpClient(timeout=timeout, follow_redirects=follow_redirects)
        return httpx_aiohttp.HttpxAiohttpClient(timeout=timeout)

    if follow_redirects is not None:
        return httpx.AsyncClient(timeout=timeout, follow_redirects=follow_redirects)
    return httpx.AsyncClient(timeout=timeout)


class AsyncAuth0:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : Auth0Environment
        The environment to use for requests from the client. from .environment import Auth0Environment



        Defaults to Auth0Environment.DEFAULT



    tenant_domain : typing.Optional[str]
        Server URL variable for 'tenantDomain'. Defaults to '{TENANT}.auth0.com'.

    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    async_token : typing.Optional[typing.Callable[[], typing.Awaitable[str]]]
        An async callable that returns a bearer token. Use this when token acquisition involves async I/O (e.g., refreshing tokens via an async HTTP client). When provided, this is used instead of the synchronous token for async requests.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    max_retries : typing.Optional[int]
        The default maximum number of retries for failed requests. Defaults to 2. Per-request `max_retries` in `request_options` takes precedence over this value.

    stream_reconnection_enabled : typing.Optional[bool]
        Whether to automatically reconnect on stream disconnection for resumable streaming endpoints. Defaults to True. Per-request `stream_reconnection_enabled` in `request_options` takes precedence over this value.

    max_stream_reconnection_attempts : typing.Optional[int]
        The maximum number of reconnection attempts for resumable streaming endpoints. Defaults to no limit. Per-request `max_stream_reconnection_attempts` in `request_options` takes precedence over this value.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    logging : typing.Optional[typing.Union[LogConfig, Logger]]
        Configure logging for the SDK. Accepts a LogConfig dict with 'level' (debug/info/warn/error), 'logger' (custom logger implementation), and 'silent' (boolean, defaults to True) fields. You can also pass a pre-configured Logger instance.

    Examples
    --------
    from auth0 import AsyncAuth0

    client = AsyncAuth0(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: Auth0Environment = Auth0Environment.DEFAULT,
        tenant_domain: typing.Optional[str] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        async_token: typing.Optional[typing.Callable[[], typing.Awaitable[str]]] = None,
        timeout: typing.Optional[float] = None,
        max_retries: typing.Optional[int] = None,
        stream_reconnection_enabled: typing.Optional[bool] = None,
        max_stream_reconnection_attempts: typing.Optional[int] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        _defaulted_max_retries = max_retries if max_retries is not None else 2
        if tenant_domain is not None:
            _tenant_domain = tenant_domain if tenant_domain is not None else "{TENANT}.auth0.com"
            base_url = "https://{tenantDomain}/api/v2".format(tenantDomain=_tenant_domain)
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            headers=headers,
            async_token=async_token,
            httpx_client=httpx_client
            if httpx_client is not None
            else _make_default_async_client(timeout=_defaulted_timeout, follow_redirects=follow_redirects),
            timeout=_defaulted_timeout,
            max_retries=_defaulted_max_retries,
            stream_reconnection_enabled=stream_reconnection_enabled,
            max_stream_reconnection_attempts=max_stream_reconnection_attempts,
            logging=logging,
        )
        self._actions: typing.Optional[AsyncActionsClient] = None
        self._branding: typing.Optional[AsyncBrandingClient] = None
        self._client_grants: typing.Optional[AsyncClientGrantsClient] = None
        self._clients: typing.Optional[AsyncClientsClient] = None
        self._connection_profiles: typing.Optional[AsyncConnectionProfilesClient] = None
        self._connections: typing.Optional[AsyncConnectionsClient] = None
        self._custom_domains: typing.Optional[AsyncCustomDomainsClient] = None
        self._device_credentials: typing.Optional[AsyncDeviceCredentialsClient] = None
        self._email_templates: typing.Optional[AsyncEmailTemplatesClient] = None
        self._event_streams: typing.Optional[AsyncEventStreamsClient] = None
        self._events: typing.Optional[AsyncEventsClient] = None
        self._flows: typing.Optional[AsyncFlowsClient] = None
        self._forms: typing.Optional[AsyncFormsClient] = None
        self._user_grants: typing.Optional[AsyncUserGrantsClient] = None
        self._groups: typing.Optional[AsyncGroupsClient] = None
        self._hooks: typing.Optional[AsyncHooksClient] = None
        self._jobs: typing.Optional[AsyncJobsClient] = None
        self._log_streams: typing.Optional[AsyncLogStreamsClient] = None
        self._logs: typing.Optional[AsyncLogsClient] = None
        self._network_acls: typing.Optional[AsyncNetworkAclsClient] = None
        self._organizations: typing.Optional[AsyncOrganizationsClient] = None
        self._prompts: typing.Optional[AsyncPromptsClient] = None
        self._rate_limit_policies: typing.Optional[AsyncRateLimitPoliciesClient] = None
        self._refresh_tokens: typing.Optional[AsyncRefreshTokensClient] = None
        self._resource_servers: typing.Optional[AsyncResourceServersClient] = None
        self._roles: typing.Optional[AsyncRolesClient] = None
        self._rules: typing.Optional[AsyncRulesClient] = None
        self._rules_configs: typing.Optional[AsyncRulesConfigsClient] = None
        self._self_service_profiles: typing.Optional[AsyncSelfServiceProfilesClient] = None
        self._sessions: typing.Optional[AsyncSessionsClient] = None
        self._stats: typing.Optional[AsyncStatsClient] = None
        self._supplemental_signals: typing.Optional[AsyncSupplementalSignalsClient] = None
        self._tickets: typing.Optional[AsyncTicketsClient] = None
        self._token_exchange_profiles: typing.Optional[AsyncTokenExchangeProfilesClient] = None
        self._user_attribute_profiles: typing.Optional[AsyncUserAttributeProfilesClient] = None
        self._user_blocks: typing.Optional[AsyncUserBlocksClient] = None
        self._users: typing.Optional[AsyncUsersClient] = None
        self._anomaly: typing.Optional[AsyncAnomalyClient] = None
        self._attack_protection: typing.Optional[AsyncAttackProtectionClient] = None
        self._emails: typing.Optional[AsyncEmailsClient] = None
        self._guardian: typing.Optional[AsyncGuardianClient] = None
        self._keys: typing.Optional[AsyncKeysClient] = None
        self._risk_assessments: typing.Optional[AsyncRiskAssessmentsClient] = None
        self._tenants: typing.Optional[AsyncTenantsClient] = None
        self._verifiable_credentials: typing.Optional[AsyncVerifiableCredentialsClient] = None

    @property
    def actions(self):
        if self._actions is None:
            from .actions.client import AsyncActionsClient  # noqa: E402

            self._actions = AsyncActionsClient(client_wrapper=self._client_wrapper)
        return self._actions

    @property
    def branding(self):
        if self._branding is None:
            from .branding.client import AsyncBrandingClient  # noqa: E402

            self._branding = AsyncBrandingClient(client_wrapper=self._client_wrapper)
        return self._branding

    @property
    def client_grants(self):
        if self._client_grants is None:
            from .client_grants.client import AsyncClientGrantsClient  # noqa: E402

            self._client_grants = AsyncClientGrantsClient(client_wrapper=self._client_wrapper)
        return self._client_grants

    @property
    def clients(self):
        if self._clients is None:
            from .clients.client import AsyncClientsClient  # noqa: E402

            self._clients = AsyncClientsClient(client_wrapper=self._client_wrapper)
        return self._clients

    @property
    def connection_profiles(self):
        if self._connection_profiles is None:
            from .connection_profiles.client import AsyncConnectionProfilesClient  # noqa: E402

            self._connection_profiles = AsyncConnectionProfilesClient(client_wrapper=self._client_wrapper)
        return self._connection_profiles

    @property
    def connections(self):
        if self._connections is None:
            from .connections.client import AsyncConnectionsClient  # noqa: E402

            self._connections = AsyncConnectionsClient(client_wrapper=self._client_wrapper)
        return self._connections

    @property
    def custom_domains(self):
        if self._custom_domains is None:
            from .custom_domains.client import AsyncCustomDomainsClient  # noqa: E402

            self._custom_domains = AsyncCustomDomainsClient(client_wrapper=self._client_wrapper)
        return self._custom_domains

    @property
    def device_credentials(self):
        if self._device_credentials is None:
            from .device_credentials.client import AsyncDeviceCredentialsClient  # noqa: E402

            self._device_credentials = AsyncDeviceCredentialsClient(client_wrapper=self._client_wrapper)
        return self._device_credentials

    @property
    def email_templates(self):
        if self._email_templates is None:
            from .email_templates.client import AsyncEmailTemplatesClient  # noqa: E402

            self._email_templates = AsyncEmailTemplatesClient(client_wrapper=self._client_wrapper)
        return self._email_templates

    @property
    def event_streams(self):
        if self._event_streams is None:
            from .event_streams.client import AsyncEventStreamsClient  # noqa: E402

            self._event_streams = AsyncEventStreamsClient(client_wrapper=self._client_wrapper)
        return self._event_streams

    @property
    def events(self):
        if self._events is None:
            from .events.client import AsyncEventsClient  # noqa: E402

            self._events = AsyncEventsClient(client_wrapper=self._client_wrapper)
        return self._events

    @property
    def flows(self):
        if self._flows is None:
            from .flows.client import AsyncFlowsClient  # noqa: E402

            self._flows = AsyncFlowsClient(client_wrapper=self._client_wrapper)
        return self._flows

    @property
    def forms(self):
        if self._forms is None:
            from .forms.client import AsyncFormsClient  # noqa: E402

            self._forms = AsyncFormsClient(client_wrapper=self._client_wrapper)
        return self._forms

    @property
    def user_grants(self):
        if self._user_grants is None:
            from .user_grants.client import AsyncUserGrantsClient  # noqa: E402

            self._user_grants = AsyncUserGrantsClient(client_wrapper=self._client_wrapper)
        return self._user_grants

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import AsyncGroupsClient  # noqa: E402

            self._groups = AsyncGroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def hooks(self):
        if self._hooks is None:
            from .hooks.client import AsyncHooksClient  # noqa: E402

            self._hooks = AsyncHooksClient(client_wrapper=self._client_wrapper)
        return self._hooks

    @property
    def jobs(self):
        if self._jobs is None:
            from .jobs.client import AsyncJobsClient  # noqa: E402

            self._jobs = AsyncJobsClient(client_wrapper=self._client_wrapper)
        return self._jobs

    @property
    def log_streams(self):
        if self._log_streams is None:
            from .log_streams.client import AsyncLogStreamsClient  # noqa: E402

            self._log_streams = AsyncLogStreamsClient(client_wrapper=self._client_wrapper)
        return self._log_streams

    @property
    def logs(self):
        if self._logs is None:
            from .logs.client import AsyncLogsClient  # noqa: E402

            self._logs = AsyncLogsClient(client_wrapper=self._client_wrapper)
        return self._logs

    @property
    def network_acls(self):
        if self._network_acls is None:
            from .network_acls.client import AsyncNetworkAclsClient  # noqa: E402

            self._network_acls = AsyncNetworkAclsClient(client_wrapper=self._client_wrapper)
        return self._network_acls

    @property
    def organizations(self):
        if self._organizations is None:
            from .organizations.client import AsyncOrganizationsClient  # noqa: E402

            self._organizations = AsyncOrganizationsClient(client_wrapper=self._client_wrapper)
        return self._organizations

    @property
    def prompts(self):
        if self._prompts is None:
            from .prompts.client import AsyncPromptsClient  # noqa: E402

            self._prompts = AsyncPromptsClient(client_wrapper=self._client_wrapper)
        return self._prompts

    @property
    def rate_limit_policies(self):
        if self._rate_limit_policies is None:
            from .rate_limit_policies.client import AsyncRateLimitPoliciesClient  # noqa: E402

            self._rate_limit_policies = AsyncRateLimitPoliciesClient(client_wrapper=self._client_wrapper)
        return self._rate_limit_policies

    @property
    def refresh_tokens(self):
        if self._refresh_tokens is None:
            from .refresh_tokens.client import AsyncRefreshTokensClient  # noqa: E402

            self._refresh_tokens = AsyncRefreshTokensClient(client_wrapper=self._client_wrapper)
        return self._refresh_tokens

    @property
    def resource_servers(self):
        if self._resource_servers is None:
            from .resource_servers.client import AsyncResourceServersClient  # noqa: E402

            self._resource_servers = AsyncResourceServersClient(client_wrapper=self._client_wrapper)
        return self._resource_servers

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import AsyncRolesClient  # noqa: E402

            self._roles = AsyncRolesClient(client_wrapper=self._client_wrapper)
        return self._roles

    @property
    def rules(self):
        if self._rules is None:
            from .rules.client import AsyncRulesClient  # noqa: E402

            self._rules = AsyncRulesClient(client_wrapper=self._client_wrapper)
        return self._rules

    @property
    def rules_configs(self):
        if self._rules_configs is None:
            from .rules_configs.client import AsyncRulesConfigsClient  # noqa: E402

            self._rules_configs = AsyncRulesConfigsClient(client_wrapper=self._client_wrapper)
        return self._rules_configs

    @property
    def self_service_profiles(self):
        if self._self_service_profiles is None:
            from .self_service_profiles.client import AsyncSelfServiceProfilesClient  # noqa: E402

            self._self_service_profiles = AsyncSelfServiceProfilesClient(client_wrapper=self._client_wrapper)
        return self._self_service_profiles

    @property
    def sessions(self):
        if self._sessions is None:
            from .sessions.client import AsyncSessionsClient  # noqa: E402

            self._sessions = AsyncSessionsClient(client_wrapper=self._client_wrapper)
        return self._sessions

    @property
    def stats(self):
        if self._stats is None:
            from .stats.client import AsyncStatsClient  # noqa: E402

            self._stats = AsyncStatsClient(client_wrapper=self._client_wrapper)
        return self._stats

    @property
    def supplemental_signals(self):
        if self._supplemental_signals is None:
            from .supplemental_signals.client import AsyncSupplementalSignalsClient  # noqa: E402

            self._supplemental_signals = AsyncSupplementalSignalsClient(client_wrapper=self._client_wrapper)
        return self._supplemental_signals

    @property
    def tickets(self):
        if self._tickets is None:
            from .tickets.client import AsyncTicketsClient  # noqa: E402

            self._tickets = AsyncTicketsClient(client_wrapper=self._client_wrapper)
        return self._tickets

    @property
    def token_exchange_profiles(self):
        if self._token_exchange_profiles is None:
            from .token_exchange_profiles.client import AsyncTokenExchangeProfilesClient  # noqa: E402

            self._token_exchange_profiles = AsyncTokenExchangeProfilesClient(client_wrapper=self._client_wrapper)
        return self._token_exchange_profiles

    @property
    def user_attribute_profiles(self):
        if self._user_attribute_profiles is None:
            from .user_attribute_profiles.client import AsyncUserAttributeProfilesClient  # noqa: E402

            self._user_attribute_profiles = AsyncUserAttributeProfilesClient(client_wrapper=self._client_wrapper)
        return self._user_attribute_profiles

    @property
    def user_blocks(self):
        if self._user_blocks is None:
            from .user_blocks.client import AsyncUserBlocksClient  # noqa: E402

            self._user_blocks = AsyncUserBlocksClient(client_wrapper=self._client_wrapper)
        return self._user_blocks

    @property
    def users(self):
        if self._users is None:
            from .users.client import AsyncUsersClient  # noqa: E402

            self._users = AsyncUsersClient(client_wrapper=self._client_wrapper)
        return self._users

    @property
    def anomaly(self):
        if self._anomaly is None:
            from .anomaly.client import AsyncAnomalyClient  # noqa: E402

            self._anomaly = AsyncAnomalyClient(client_wrapper=self._client_wrapper)
        return self._anomaly

    @property
    def attack_protection(self):
        if self._attack_protection is None:
            from .attack_protection.client import AsyncAttackProtectionClient  # noqa: E402

            self._attack_protection = AsyncAttackProtectionClient(client_wrapper=self._client_wrapper)
        return self._attack_protection

    @property
    def emails(self):
        if self._emails is None:
            from .emails.client import AsyncEmailsClient  # noqa: E402

            self._emails = AsyncEmailsClient(client_wrapper=self._client_wrapper)
        return self._emails

    @property
    def guardian(self):
        if self._guardian is None:
            from .guardian.client import AsyncGuardianClient  # noqa: E402

            self._guardian = AsyncGuardianClient(client_wrapper=self._client_wrapper)
        return self._guardian

    @property
    def keys(self):
        if self._keys is None:
            from .keys.client import AsyncKeysClient  # noqa: E402

            self._keys = AsyncKeysClient(client_wrapper=self._client_wrapper)
        return self._keys

    @property
    def risk_assessments(self):
        if self._risk_assessments is None:
            from .risk_assessments.client import AsyncRiskAssessmentsClient  # noqa: E402

            self._risk_assessments = AsyncRiskAssessmentsClient(client_wrapper=self._client_wrapper)
        return self._risk_assessments

    @property
    def tenants(self):
        if self._tenants is None:
            from .tenants.client import AsyncTenantsClient  # noqa: E402

            self._tenants = AsyncTenantsClient(client_wrapper=self._client_wrapper)
        return self._tenants

    @property
    def verifiable_credentials(self):
        if self._verifiable_credentials is None:
            from .verifiable_credentials.client import AsyncVerifiableCredentialsClient  # noqa: E402

            self._verifiable_credentials = AsyncVerifiableCredentialsClient(client_wrapper=self._client_wrapper)
        return self._verifiable_credentials


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: Auth0Environment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventUserCreatedObjectIdentitiesItemEnterpriseProviderEnum = typing.Union[
    typing.Literal[
        "ad", "adfs", "google-apps", "ip", "office365", "oidc", "okta", "pingfederate", "samlp", "sharepoint", "waad"
    ],
    typing.Any,
]
